#!/bin/bash
#
# Genome annotation. Annotate VSGs, merge with companion annotation and transfer (alignment-based other annotations: centromeres, transcription start sites, etc.)
#
#to run ./run.sh
#
##Requirements##
#pyfastq
#The Companion annotation output file should be inside the input file

module load ncbi-blast/2.7.1+
module load ngs/bedtools2/2.26.0
source /work/project/ladsie_003/python_venv/bin/activate

main(){
	set_variables
	create_folders
	generate_blast_db
	blast_VSGs_to_genome
	blast2gff
	clean_Companion_gff
	merge_gffs
	transfer_manual_annotation
}

###############################################################################
# Setting of global variables
###############################################################################

set_variables(){
	INPUT_FOLDER=input
	OUTPUT_FOLDER=927v41_with_pseudo_output

	SCRIPTS_FOLDER=bin
	GENOME_FILE=HGAP3_Tb427v9.9_without_allelic_variants.fasta
	VSG_FASTA_FILE=VSGs_CDS_Cross_list.fasta
	BLAST_FOLDER=$OUTPUT_FOLDER/blast
	GFF_FOLDER=$OUTPUT_FOLDER/gff
	COMPANION_FILE=Companion_927v41_${GENOME_FILE%.fasta}.gff3
	GENE_ID_PREFIX="Tb427_000"
	FIRST_NEW_GENE_ID=844600 #Based on the last ID assigned by Companion + 100 #Last ID="ID=Tb427_000844500"
	OLD_GENOME_FILE=HGAP3_Tb427v9.2.fasta
	OLD_GENOME_ANNOTATION=HGAP3_Tb427v9.2.gff
}


###############################################################################
# Prelude
###############################################################################

create_folders(){
	
	mkdir -p \
	$INPUT_FOLDER \
	$SCRIPTS_FOLDER \
	$OUTPUT_FOLDER \
	$BLAST_FOLDER \
	$GFF_FOLDER

}


generate_blast_db(){

jid1=$(sbatch --job-name=blastdb --wrap="makeblastdb -in $INPUT_FOLDER/$GENOME_FILE -dbtype nucl -parse_seqids")
jid1=$(echo $jid1 | sed "s/Submitted batch job //")

}

blast_VSGs_to_genome(){

OUTPUT_FILE="${VSG_FASTA_FILE%.fasta}_to_${GENOME_FILE%.fasta}.out"

jid2=$(sbatch --job-name=runBlast --dependency=afterok:${jid1} --wrap="blastn -db $INPUT_FOLDER/$GENOME_FILE -query $INPUT_FOLDER/$VSG_FASTA_FILE -outfmt 7 -out $BLAST_FOLDER/${OUTPUT_FILE}")
jid2=$(echo $jid2 | sed "s/Submitted batch job //")

}

blast2gff(){

BLAST_OUTPUT_FILE=$BLAST_FOLDER/"${VSG_FASTA_FILE%.fasta}_to_${GENOME_FILE%.fasta}.out"

jid3=$(sbatch --job-name=blast2gff --dependency=afterok:${jid2} --wrap="perl $SCRIPTS_FOLDER/Blast_output_to_gff3_v2.pl $BLAST_OUTPUT_FILE $INPUT_FOLDER/$VSG_FASTA_FILE > $GFF_FOLDER/VSGs_in_${GENOME_FILE%.fasta}.gff3")
jid3=$(echo $jid3 | sed "s/Submitted batch job //")

jid4=$(sbatch --job-name=sort_gff --dependency=afterok:${jid3} --wrap="sort -k1,1 -k4,4n -o $GFF_FOLDER/VSGs_in_${GENOME_FILE%.fasta}.gff3 $GFF_FOLDER/VSGs_in_${GENOME_FILE%.fasta}.gff3")
jid4=$(echo $jid4 | sed "s/Submitted batch job //")
}

clean_Companion_gff(){

#Remove pseudogene entries overlapping rRNA entries and small protein-coding gene entries

INPUT=${INPUT_FOLDER}/${COMPANION_FILE}
OUTPUT=${GFF_FOLDER}/Cleaned_${COMPANION_FILE}
MIN_LENGTH=60 # Nucleotides

jid5=$(sbatch --job-name=rm_pseudo_rRNA_overlapping --export=INPUT=${INPUT},MIN_LENGTH=${MIN_LENGTH},OUTPUT_FOLDER=${GFF_FOLDER},OUTPUT=${OUTPUT} ${SCRIPTS_FOLDER}/clean_Companion_annotation.sh)
jid5=$(echo $jid5 | sed "s/Submitted batch job //")

}

merge_gffs(){

#merge VSG-overlapping entries in the BLAST result

jid6=$(sbatch --job-name=join_overlapping --dependency=afterok:${jid4} --wrap="perl $SCRIPTS_FOLDER/join_overlapping_entries_VSGs_gff3.pl $GFF_FOLDER/VSGs_in_${GENOME_FILE%.fasta}.gff3 > ${GFF_FOLDER}/VSGs_in_${GENOME_FILE%.fasta}_overlapping_joined.gff3")
jid6=$(echo $jid6 | sed "s/Submitted batch job //")


#join VSG_gff with Companion_gff, remove comment lines and sort them

jid7=$(sbatch --job-name=join_VSG_and_Companion_gffs --dependency=afterok:${jid6}:${jid5} --wrap="cat ${GFF_FOLDER}/VSGs_in_${GENOME_FILE%.fasta}_overlapping_joined.gff3 ${GFF_FOLDER}/Cleaned_${COMPANION_FILE} | grep -v '^#' | sort -k1,1 -k4,4n > ${GFF_FOLDER}/joined_VSG_Companion_in_${GENOME_FILE%.fasta}.gff3")
jid7=$(echo $jid7 | sed "s/Submitted batch job //")


#merge VSG entries with Companion_gff entries

jid8=$(sbatch --job-name=merge_VSG_and_gene_Companion_entries --dependency=afterok:${jid7} --wrap="perl $SCRIPTS_FOLDER/merge_VSG_Companion_gff3.pl ${GFF_FOLDER}/joined_VSG_Companion_in_${GENOME_FILE%.fasta}.gff3 > ${GFF_FOLDER}/merged_VSG_Companion_in_${GENOME_FILE%.fasta}.gff3")
jid8=$(echo $jid8 | sed "s/Submitted batch job //")


#add geneID to VSG entries from the BLAST result that were not merged with Companion VSG results.

jid9=$(sbatch --job-name=Add_GeneIDs_orphan_VSG_entries --dependency=afterok:${jid8} --wrap="perl $SCRIPTS_FOLDER/Add_IDs_to_VSGs.pl ${GFF_FOLDER}/merged_VSG_Companion_in_${GENOME_FILE%.fasta}.gff3 $GENE_ID_PREFIX $FIRST_NEW_GENE_ID > ${GFF_FOLDER}/Complete_${GENOME_FILE%.fasta}.gff3 ")
jid9=$(echo $jid9 | sed "s/Submitted batch job //")

##re-add the header from the Companion gff file

jid10=$(sbatch --job-name=Add-comments --dependency=afterok:${jid9} --wrap="grep '^##' ${INPUT_FOLDER}/${COMPANION_FILE} | grep -v '^###' | cat - ${GFF_FOLDER}/Complete_${GENOME_FILE%.fasta}.gff3 > ${GFF_FOLDER}/Full_${GENOME_FILE%.fasta}.gff3")
jid10=$(echo $jid10 | sed "s/Submitted batch job //")

}

transfer_manual_annotation(){


ANNOTATION_IN="Full_${GENOME_FILE%.fasta}.gff3"

jid11=$(sbatch --job-name=transfer-manual-annotation --dependency=afterok:${jid10} --export=INPUT_FOLDER=${INPUT_FOLDER},OUTPUT_FOLDER=${OUTPUT_FOLDER},OLD_GENOME_ANNOTATION=${OLD_GENOME_ANNOTATION},OLD_GENOME_FILE=${OLD_GENOME_FILE},SCRIPTS_FOLDER=${SCRIPTS_FOLDER},GENOME_FILE=${GENOME_FILE},GFF_FOLDER=${GFF_FOLDER},ANNOTATION_IN=${ANNOTATION_IN} ${SCRIPTS_FOLDER}/transfer_manual_annotation.sh)
jid11=$(echo $jid11 | sed "s/Submitted batch job //")

}

main

